var reactVueTemplateParser = require('./compiler');
const traverse = require('babel-traverse');
const { SourceMapConsumer } = require('source-map');

function sourceMapAstInPlace(tsMap, babelAst) {
  const tsConsumer = new SourceMapConsumer(tsMap);
  traverse.default.cheap(babelAst, node => {
    if (node.loc) {
      const originalStart = tsConsumer.originalPositionFor(node.loc.start);
      if (originalStart.line) {
        node.loc.start.line = originalStart.line;
        node.loc.start.column = originalStart.column;
      }
      const originalEnd = tsConsumer.originalPositionFor(node.loc.end);
      if (originalEnd.line) {
        node.loc.end.line = originalEnd.line;
        node.loc.end.column = originalEnd.column;
      }
    }
  });
}

function transform({ src, filename, options, upstreamTransformer }) {
  if (typeof src === 'object') {
    // handle RN >= 0.46
    ({ src, filename, options } = src);
  }
  const outputFile = reactVueTemplateParser(src, filename, options);

  if (!outputFile.output) {
    return upstreamTransformer.transform({
      src: outputFile,
      filename,
      options
    });
  } else {
    // Source Map support
    const babelCompileResult = upstreamTransformer.transform({
      src: outputFile.output,
      filename,
      options
    });
    if (outputFile.mappings) {
      sourceMapAstInPlace(outputFile.mappings, babelCompileResult.ast);
    }
    return babelCompileResult;
  }
}

module.exports = transform;
